#!/usr/bin/perl -w
#############################################################################
# PROGRAM
#     dwit
#
# SYNOPSIS
#     dwit [OPTIONS]
#
# USAGE
#     Light weight task management system.
#
# PARAMETERS
#     None
#
# OPTIONS
#     None
#
# MODULES/FILES REQUIRED
#     File::Basename
#     DBI
#
# PROGRAMS REQUIRED
#     None
#
# AUTHOR
#     Steve Dickinson
#
# DEVELOPMENT DATES
#     10.02.2010 - 10.05.2010
#
# RELEASE DATE
#
# COMMENTS
#     None
#
# REVISIONS
#     None
#############################################################################
# INTERRUPT SIGNALS
#############################################################################
$SIG{'INT' } = 'clean_up'; $SIG{'HUP' } = 'clean_up';
$SIG{'QUIT'} = 'clean_up'; $SIG{'TRAP'} = 'clean_up';
$SIG{'ABRT'} = 'clean_up'; $SIG{'STOP'} = 'clean_up';
$SIG{'TSTP'} = 'clean_up';

#############################################################################
# INCLUDE
#############################################################################
use DBI;
use File::Basename;
use strict;

#############################################################################
# DATABASE CONNECTION
#############################################################################
my $prog_path = dirname($0);
my $dwit_dbname = "dwit.db";
my $dwit_db_path = $prog_path . "/" . $dwit_dbname;

my $dbargs = {AutoCommit => 0,
              PrintError => 1};

my $dbh = DBI->connect("dbi:SQLite:dbname=$dwit_db_path","","",$dbargs);

#############################################################################
# USAGE
#############################################################################
my $prog       = basename($0,"");

my $priority_levels = priority_get_levels();

sub usage_short {

my $usage_short = <<END;
usage: $prog [OPTIONS]

$prog [-a] [-b 'body'] [-c 'category'] [-d yyyy.mm.dd] [-p number] [-p0] [-t 'title']
$prog [-C] [-i number]
$prog [-e] [-i number] [[-b 'body'] [-c 'category'] [-d yyyy.mm.dd] [-p number] [-p0] [-t 'title']]
$prog [-l] 
$prog [-la] 
$prog [-lc] 
$prog [-r] [-i number]
$prog [-U] [-i number]

END
 
    return $usage_short;
}

sub usage_short_print {
    my $usage_short = usage_short();
    print $usage_short;
    print "\n";
    exit 1;
}

sub usage {
 
my $usage_short = usage_short();
my $usage = <<END;

$usage_short
Options:
        -a,   --add                               add a task
        -b,   --body        '[text]'              task body (additional information about task)
        -c,   --category    '[text]'              task category (a way to group tasks)
        -C,   --complete                          mark task as complete
        -d,   --due         [yyyy.mm.dd]          date task is to be completed by (default: none)    
        -e,   --edit                              edit task    
        -h,   --help                              usage screen  
        -i,   --id          [id number]           id number of task    
        -l,   --list                              list tasks that are not complete
        -la,  --lista                             list all tasks
        -lc,  --listc                             list tasks that are complete
        -p,   --priority    [priority number]     task priority number
        -p0,  --prioritytop [priority number]     designate a task as top priority
        -r,   --remove                            remove a task
        -t,   --title       '[text]'              task title
        -U,   --uncomplete                        mark task as uncomplete


Action Options:
        -a    add
        -C    complete
        -e    edit
        -h    help
        -l    list
        -la   lista
        -lc   listc
        -p    priority
        -p0   prioritytop
        -r    remove
        -U    uncomplete


Informational Options:
        -b    body
        -c    category
        -d    due date
        -i    id
        -t    title


Priority Levels:
$priority_levels

Examples
--------
$prog -a -t 'new task' -b 'This task is a very important' -c 'category 1' -p 1
        This command will add a new task to the database with a title, additional information about it, a category and give it the 'high' priority level.
 
$prog -C -i 102
        This command will complete the task with an id number of 102.
 
$prog -U -i 102
        This command will uncomplete the task with an id number of 102.
 
$prog -l 
        This command will list all of the current tasks in the database that are not complete.
 
$prog -la 
        This command will list all of the current tasks in the database.

$prog -lc 
        This command will list all of the current tasks in the database that are complete.
 
$prog -e -i 55 -t 'foo bar'
        This command will change the title for task record 55 to 'foo bar'.

$prog -e -i 74 -p0
        This command will give the task with an id number of 74 the top priority level.
 
$prog -r -i 7
        This command will remove the task with an id number of 7.
 

END
 
    print $usage;
    print "\n";
 
    exit(1);
}
 
#############################################################################
# DO SOME PRELIMINARY CHECKS
#############################################################################
if (! -e "$dwit_db_path") {
    print "$prog: Could not find the dwit.db database file! It must be installed in the same directory as the dwit executable.\n\n";
    exit(1);
}

#############################################################################
# PROCESS ARGUMENTS
#############################################################################
my $opt 		= "";
my $arg_add	 	= "false";
my $arg_body	 	= "false";
my $opt_body	 	= "";
my $arg_category 	= "false";
my $opt_category 	= "";
my $arg_due_date	= "false";
my $opt_due_date	= "None";
my $arg_edit   		= "false";
my $arg_id		= "false";
my $opt_id		= "";
my $arg_list 		= "false";
my $arg_lista 		= "false";
my $arg_listc 		= "false";
my $arg_priority 	= "false";
my $opt_priority 	= "9";
my $arg_prioritytop 	= "false";
my $arg_remove 		= "false";
my $arg_title 		= "false";
my $opt_title 		= "";
my $arg_complete 	= "false";
my $arg_uncomplete 	= "false";
my $opt_complete 	= "0";
my $arg_action		= "false";
my $current_year	= "";
my $current_month	= "";
my $current_day		= "";

for (my $argc = 0; $argc <= $#ARGV; $argc++) {
    if ($ARGV[$argc]) {
        $opt =  $ARGV[$argc];
        $opt =~ s/--//; # Get rid of 2 dashes
        $opt =~ s/-//; # Get rid of 1 dash
    }
    if ($opt eq 'a' || $opt eq 'add') {
	$arg_add = "true";
        $arg_action = "true";
    } elsif ($opt eq 'b' || $opt eq 'body') {
	$arg_body = "true";
	$opt_body = $ARGV[++$argc];
        if (!$opt_body) {
	    print "$prog: argument required with body option\n\n";
	    usage_short_print();
        }
    } elsif ($opt eq 'c' || $opt eq 'category') {
	$arg_category = "true";
	$opt_category = $ARGV[++$argc];
        if (!$opt_category) {
	    print "$prog: argument required with category option\n\n";
	    usage_short_print();
        }
    } elsif ($opt eq 'C' || $opt eq 'complete') {
	$arg_complete = "true";
	$opt_complete = "1";
        $arg_action = "true";
    } elsif ($opt eq 'd' || $opt eq 'due') {
	$arg_due_date = "true";
	$opt_due_date = $ARGV[++$argc];
        if (!$opt_due_date) {
	    print "$prog: argument required with due date option\n\n";
            usage_short_print();
        }
    } elsif ($opt eq 'e' || $opt eq 'edit') {
	$arg_edit = "true";
        $arg_action = "true";
    } elsif ($opt eq 'h' || $opt eq 'help') {
	usage();
	exit 0;
    } elsif ($opt eq 'i' || $opt eq 'id') {
	$arg_id = "true";
	$opt_id = $ARGV[++$argc];
        if (!$opt_id) {
	    print "$prog: argument required with id option\n\n";
            usage_short_print();
        }
    } elsif ($opt eq 'l' || $opt eq 'list') {
	$arg_list = "true";
        $arg_action = "true";
    } elsif ($opt eq 'la' || $opt eq 'lista') {
	$arg_lista = "true";
        $arg_action = "true";
    } elsif ($opt eq 'lc' || $opt eq 'listc') {
	$arg_listc = "true";
        $arg_action = "true";
    } elsif ($opt eq 'p' || $opt eq 'priority') {
	$arg_priority = "true";
	$opt_priority = $ARGV[++$argc]; if (!$opt_priority && $opt_priority != 0) {
	    print "$prog: argument required with priority option\n\n";
            usage_short_print();
        }
    } elsif ($opt eq 'p0' || $opt eq 'prioritytop') {
	$arg_prioritytop = "true";
	$opt_priority = 0;
    } elsif ($opt eq 'r' || $opt eq 'remove') {
	$arg_remove = "true";
        $arg_action = "true";
    } elsif ($opt eq 't' || $opt eq 'title') {
	$arg_title = "true";
        $opt_title = $ARGV[++$argc];
        if (!$opt_title) {
	    print "$prog: argument required with title option\n\n";
            usage_short_print();
        }
    } elsif ($opt eq 'U' || $opt eq 'uncomplete') {
	$arg_uncomplete = "true";
	$opt_complete = "0";
        $arg_action = "true";
    } else {
	usage();
	exit 1;
    }
}

# add option must have title option
if ($arg_add eq "true") {
    if ($arg_title eq "false") {
        print "$prog: add option must include title option\n\n";
        usage_short_print();
    }
    if ($arg_complete eq "true" || $arg_edit eq "true" || $arg_list eq "true" || 
        $arg_lista eq "true" || $arg_listc eq "true" || $arg_remove eq "true" || 
        $arg_uncomplete eq "true") {
        print "$prog: invalid option given\n\n";
        usage_short_print();
    }
}
if ($arg_edit eq "true") {
    if ($arg_id eq "false") {
        print "$prog: edit option must include id option\n\n";
        usage_short_print();
    }
    if ($arg_body eq "false" && $arg_category eq "false" && $arg_due_date eq "false" && 
        $arg_priority eq "false" && $arg_prioritytop eq "false" && $arg_title eq "false") {
        print "$prog: missing option\n\n";
        usage_short_print();
    }
    if ($arg_add eq "true" || $arg_complete eq "true" || $arg_list eq "true" || 
        $arg_lista eq "true" || $arg_listc eq "true" || $arg_remove eq "true" || 
        $arg_uncomplete eq "true") {
        print "$prog: invalid option given\n\n";
        usage_short_print();
    }
}
if (($arg_list eq "true" || $arg_lista eq "true" || $arg_listc eq "true") && $#ARGV > 1 ) {
    print "$prog: options -l, -la, -lc must be used exclusively\n\n";
    usage_short_print();
}
if ($arg_complete eq "true" || $arg_uncomplete eq "true") {
    if ($arg_id eq "false") {
        print "$prog: options -C and -U must include the -i option\n\n";
        usage_short_print();
    }
    if ($#ARGV > 2) {
        print "$prog: invalid option given\n\n";
        usage_short_print();
    }
}
if ($arg_remove eq "true") {
    if ($arg_id eq "false") {
        print "$prog: option -r must include the -i option\n\n";
        usage_short_print();
    }
    if ($#ARGV > 2) {
        print "$prog: invalid option given\n\n";
        usage_short_print();
    }
}
if ($arg_action eq "false") {
    print "$prog: no action option specified\n\n";
    usage_short_print();
}

#############################################################################
# GLOBAL VARIABLES / SETTINGS
#############################################################################
umask(0);

my $user = getlogin || getpwuid($<);

#my $now = localtime;
my ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) = localtime;
$mon += 1;
$year += 1900;

my $task_title 		= "";
my $task_body	 	= "";
my $task_category 	= "";
my $task_add_date 	= $year.".".sprintf("%0.2d", $mon).".".sprintf("%0.2d", $mday);
my $task_due_date 	= "None";
my $task_priority_id 	= 9;
my $task_complete 	= 0;
my @priority_numbers;
my $priority_exists	= "";

#############################################################################
# MAIN
#############################################################################

#============================================================================
# Execute action options
#============================================================================
if ($arg_add eq "true") {
    priority_number_check();
    $task_priority_id = priority_get_id("$opt_priority");
#    priority_top_check($task_priority_id);
    task_add("$opt_title", "$opt_body", "$opt_category", "$task_add_date", "$opt_due_date", "$task_priority_id", "$opt_complete");
} elsif ($arg_list eq "true") {
    task_list("asc", "c");
} elsif ($arg_lista eq "true") {
    task_list("asc", "");
} elsif ($arg_listc eq "true") {
    task_list("asc", "nc");
} elsif ($arg_complete eq "true") {
    task_complete("$opt_id");
} elsif ($arg_uncomplete eq "true") {
    task_uncomplete("$opt_id");
} elsif ($arg_remove eq "true") {
    task_remove("$opt_id");
} elsif ($arg_edit eq "true") {
    if ($arg_body eq "true") {
        body_update("$opt_id", "$opt_body");
    }
    if ($arg_category eq "true") {
        category_update("$opt_id", "$opt_category");
    }
    if ($arg_due_date eq "true") {
        due_date_update("$opt_id", "$opt_due_date");
    }
    if ($arg_title eq "true") {
        title_update("$opt_id", "$opt_title");
    }
    if ($arg_priority eq "true") {
        priority_number_check();
        $task_priority_id = priority_get_id("$opt_priority");
        priority_update("$opt_id", "$task_priority_id");
    } 
    if ($arg_prioritytop eq "true") {
        $task_priority_id = priority_get_id("0");
        priority_update("$opt_id", "$task_priority_id");
    }
}

$dbh->disconnect();
exit 0;

#============================================================================
# Functions
#============================================================================
sub task_add {
    my ($task_title, $task_body, $task_category, $task_submitted_date, $task_due_date, $task_priority_id, $task_complete) = @_;

    $dbh->do( "INSERT INTO tasks VALUES ( NULL, '$task_title', '$task_body', '$task_category', '$task_submitted_date', '$task_due_date', '$task_priority_id', '$task_complete' ) " );

    if ($dbh->err()) { die "$DBI::errstr\n"; }

    $dbh->commit();
}

sub task_complete {
    my ($task_id) = @_;

    $dbh->do( "UPDATE tasks SET task_complete=1 WHERE task_id=$task_id " );

    if ($dbh->err()) { die "$DBI::errstr\n"; }

    $dbh->commit();
}

sub task_uncomplete {
    my ($task_id) = @_;

    $dbh->do( "UPDATE tasks SET task_complete=0 WHERE task_id=$task_id " );

    if ($dbh->err()) { die "$DBI::errstr\n"; }

    $dbh->commit();
}

sub task_remove {
    my ($task_id) = @_;

    $dbh->do( "DELETE FROM tasks WHERE task_id=$task_id " );

    if ($dbh->err()) { die "$DBI::errstr\n"; }

    $dbh->commit();
}

sub task_list {
    my ($order, $filter) = @_;

    my $order_sql = "ASC";
    my $filter_sql = "";
    if ($order eq "asc") {
	$order_sql = "ASC";
    }
    if ($order eq "desc") {
	$order_sql = "DESC";
    }
    if ($filter eq "nc") {
	$filter_sql = "WHERE t1.task_complete = 1";
    }
    if ($filter eq "c") {
	$filter_sql = "WHERE t1.task_complete = 0";
    }
    my ($task_priority_number, $task_priority_name, $task_id, $task_submitted_date, $task_due_date, $task_complete, $task_completed, $task_title, $task_body, $task_category, $task_title_task_body);
    my $task_id_length 			= 0;
    my $task_id_length_save 		= 0;
    my $task_id_width 			= 0;
    my $task_title_length 		= 0;
    my $task_title_length_save 		= 0;
    my $task_title_width 		= 0;
    my $task_category_length 		= 0;
    my $task_category_length_save 	= 0;
    my $task_category_width 		= 0;

    my $res = $dbh->selectall_arrayref( qq( SELECT task_id, task_title, task_category from tasks ) );

    foreach( @$res ) {
        $task_id 		= $_->[0];
        $task_title 		= $_->[1];
        $task_category 		= $_->[2];
        $task_id_length		= length($task_id);
        $task_title_length	= length($task_title);
        $task_category_length	= length($task_category);
        if ($task_id_length > $task_id_length_save) {
            $task_id_length_save = $task_id_length;
        }
        if ($task_title_length > $task_title_length_save) {
            $task_title_length_save = $task_title_length;
        }
        if ($task_category_length > $task_category_length_save) {
            $task_category_length_save = $task_category_length;
        }
    }

    if ($task_id_length_save > 3) {
        $task_id_width = 2 + ($task_id_length_save - 3);
    } else {
        $task_id_width = 1;
    }

    if ($task_title_length_save > 10) {
        $task_title_width = 2 + ($task_title_length_save - 3);
    } else {
        $task_title_width = 9;
    }

    if ($task_category_length_save > 8) {
        $task_category_width = 2 + ($task_category_length_save - 3);
    } else {
        $task_category_width = 7;
    }

    my $print_format = "format REPORT =\n".
		       "@  @<<<<<  @".">" x $task_id_width ."  @<<<<<<<<<  @<<<<<<<<<  @  @"."<" x $task_category_width ."  @"."<" x $task_title_width ."  @*\n".
		       "\$task_priority_number, \$task_priority_name, \$task_id, \$task_due_date, \$task_submitted_date, \$task_completed, \$task_category, \$task_title, \$task_body\n".
		       ".\n";

    eval $print_format;
    $~ = "REPORT";

    $res = $dbh->selectall_arrayref( qq( SELECT t2.priority_number, t2.priority_name, t1.task_id, t1.task_title, t1.task_body, t1.task_category, t1.task_submitted_date, t1.task_due_date, t1.task_complete FROM tasks as t1 INNER JOIN priorities as t2 ON t1.task_priority_id = t2.priority_id ${filter_sql} ORDER BY t2.priority_number $order_sql ) );

    $task_priority_number 	= "^\n=";
    $task_priority_name 	= "LEVEL";
    $task_id 			= "ID";
    $task_submitted_date 	= "ADD DATE";
    $task_due_date 		= "DUE DATE";
    $task_completed		= "X";
    $task_category		= "CATEGORY";
    $task_title			= "TASK TITLE";
    $task_body			= "TASK BODY";
    write;

    foreach( @$res ) {
        $task_priority_number 	= $_->[0];
        $task_priority_name 	= $_->[1];
        $task_id 		= $_->[2];
        $task_title 		= $_->[3];
        $task_body 		= $_->[4];
        $task_category 		= $_->[5];
        $task_submitted_date 	= $_->[6];
        $task_due_date 		= $_->[7];
        $task_complete 		= $_->[8];
	$task_completed		= "";
        if ($task_complete == 1) {
            $task_completed = "X";
        }
#        $task_title_task_body = $task_title . ": " . $task_body;
        write;
    }
}

sub body_update {
    my ($task_id, $opt_body) = @_;     

    $dbh->do( "UPDATE tasks SET task_body='$opt_body' WHERE task_id=$task_id " );
    if ($dbh->err()) { die "$DBI::errstr\n"; }
    $dbh->commit();
}

sub category_update {
    my ($task_id, $opt_category) = @_;     

print "$opt_category";
    $dbh->do( "UPDATE tasks SET task_category='$opt_category' WHERE task_id=$task_id " );
    if ($dbh->err()) { die "$DBI::errstr\n"; }
    $dbh->commit();
}

sub due_date_update {
    my ($task_id, $opt_due_date) = @_;     

    $dbh->do( "UPDATE tasks SET task_due_date='$opt_due_date' WHERE task_id=$task_id " );
    if ($dbh->err()) { die "$DBI::errstr\n"; }
    $dbh->commit();
}

sub title_update {
    my ($task_id, $opt_title) = @_;     

    $dbh->do( "UPDATE tasks SET task_title='$opt_title' WHERE task_id=$task_id " );
    if ($dbh->err()) { die "$DBI::errstr\n"; }
    $dbh->commit();
}

sub priority_update {
    my ($task_id, $priority_id) = @_;     

#    priority_top_check($priority_id);

    $dbh->do( "UPDATE tasks SET task_priority_id=$priority_id WHERE task_id=$task_id " );
    if ($dbh->err()) { die "$DBI::errstr\n"; }
    $dbh->commit();
}

sub priority_get_id {
    my ($priority_number) = @_;

    my @res = $dbh->selectrow_array( qq( SELECT * from priorities WHERE priority_number = $priority_number ) );

    return $res[0];
}

sub priority_get_numbers {
    my ($priority_number, @priority_array);

    my $res = $dbh->selectall_arrayref( qq( SELECT priority_number from priorities ) );

    foreach( @$res ) {
        $priority_number 	= $_->[0];
        push (@priority_array, $priority_number);
    }

    return @priority_array;
}

sub priority_number_check {
    my (@priority_numbers);

    @priority_numbers = priority_get_numbers();
    if (!exists {map { $_ => 1 } @priority_numbers}->{$opt_priority}) {
        print "$prog: priority number entered does not exist!\n\n";
        exit(1);
    }
}

sub priority_top_check {
    my ($priority_id) = @_;

    my @res = $dbh->selectrow_array( q( SELECT priority_id FROM priorities WHERE priority_number = 0 ) );
    my $table_top_priority_id = $res[0];

    @res = $dbh->selectrow_array( q( SELECT COUNT(*) FROM tasks as t1 INNER JOIN priorities as t2 ON t1.task_priority_id = t2.priority_id WHERE t2.priority_number = 0 ) );
    my $top_priority_count = $res[0];
    if ($priority_id == $table_top_priority_id && $top_priority_count > 0) {
        print "\n$prog: Can only have one top priority task at a time!\n\n";
        exit(1);
    }
}

sub priority_get_levels {
    my ($priority_number, $priority_name, $priority_string);

    my $res = $dbh->selectall_arrayref( qq( SELECT * from priorities ORDER BY priority_number ASC ) );

    foreach( @$res ) {
        $priority_number 	= $_->[1];
        $priority_name 		= $_->[2];
	$priority_string       .= "        " . $priority_number . "     " . $priority_name . "\n";
    }

    return $priority_string;
}

#############################################################################
# CLEAN UP
#############################################################################
sub clean_up {
    exit 0;
}

